import os
from typing import Literal

from utils.exceptions import MissingEnvVariable


EnvName = Literal["dev", "staging", "prod"]

def get_env_name() -> EnvName:
    env = os.getenv("ENV")
    
    if env is None:
        raise MissingEnvVariable("Could not get 'ENV' env variable, therefore can't deduce logging_level.")
    elif env in ["dev","staging", "prod"]:
        return env
    else:
        raise ValueError(
            f"'ENV' is set to: {env}. This is not a supported env! Please choose between: [dev, staging, prod]"
        )



import os
import rabbitpy
from typing import List, Tuple

from inoopa_logging import create_logger
from utils.exceptions import MissingEnvVariable

_logger = create_logger("INOOPA.UTILS.RABBITMQ_HELPERS")


def _get_credentials_from_env() -> Tuple[str, str, str]:
    env_variables = {
        "queue_host": os.getenv("QUEUE_HOST"),
        "queue_username": os.getenv("QUEUE_USERNAME"),
        "queue_password": os.getenv("QUEUE_PASSWORD"),
    }

    missing_variables = []
    for key, value in env_variables.items():
        if value is None:
            _logger.error(f"QUEUE env variable missing: {key}")
            missing_variables.append(key)
    if len(missing_variables) > 0:
        raise MissingEnvVariable(f"Missing env variables: {missing_variables}")

    return env_variables["queue_host"], env_variables['queue_username'], env_variables['queue_password']

def _get_queue_name_with_env(queue_name: str) -> str:
    """Helper function to add the env name to the queue name."""
    env_name = os.getenv("ENV")
    if env_name is None:
        raise MissingEnvVariable("Missing env variable: ENV")
    return f"{queue_name}_{env_name}"

def get_messages_from_queue(queue_name: str, queue_batch_size: int = 1) -> List[str]:
    """Helper function to get X messages from a rabbitMQ queue."""
    host, username, password = _get_credentials_from_env()
    queue_name_with_env = _get_queue_name_with_env(queue_name)
    messages_body = []
    _logger.info(f"Connecting to queue: {queue_name_with_env}...")
    with rabbitpy.Connection(f"amqp://{username}:{password}@{host}/%2f") as conn:
        with conn.channel() as channel:
            queue = rabbitpy.Queue(channel, queue_name_with_env, durable=True)
            queue.declare()
            _logger.info("Connected to queue, reading messages...")
            for i in range(queue_batch_size):
                message = queue.get(acknowledge=True)
                if message:
                    messages_body.append(message.body.decode())
                    # Tell the queue that this message has been read and should be removed from queue
                    message.ack()
                    _logger.info(f"Message {i+1}/{queue_batch_size} read")
                else:
                    _logger.info("Queue empty, stoping...")
                    break
    return messages_body

def push_to_queue(queue_name: str, message: str) -> None:
    """Helper function to push a message to a rabbitMQ queue."""
    host, username, password = _get_credentials_from_env()
    queue_name_with_env = _get_queue_name_with_env(queue_name)

    _logger.info(f"Connecting to queue: {queue_name_with_env}...")
    with rabbitpy.Connection(f"amqp://{username}:{password}@{host}/%2f") as conn:
        with conn.channel() as channel:
            queue = rabbitpy.Queue(channel, queue_name_with_env, durable=True)
            queue.declare()
            
            # Create a new message
            msg = rabbitpy.Message(channel, message)
            
            # Publish the message to the specified queue
            msg.publish('', queue_name_with_env)
            _logger.info(f"Message pushed to {queue_name_with_env}.")

def empty_queue(queue_name: str) -> None:
    """
    Helper function to remove all messages from a RabbitMQ queue.
    """
    host, username, password = _get_credentials_from_env()
    queue_name = _get_queue_name_with_env(queue_name)
    _logger.info(f"Purging queue: {queue_name}")
    with rabbitpy.Connection(f"amqp://{username}:{password}@{host}/%2f") as conn:
        with conn.channel() as channel:
            queue = rabbitpy.Queue(channel, queue_name, durable=True)
            queue.declare()
            message_count = queue.purge()
        _logger.info(f"Removed {message_count} messages from queue: {queue_name}")
        _logger.info("Queue empty!")


"""Base Inoopa configuration for python Logging."""
import logging
from typing import Literal
from datetime import datetime

from utils.env_variables_helper import get_env_name


LoggingLevel = Literal["CRITICAL","FATAL","ERROR","WARN","WARNING","INFO","DEBUG"]

def create_logger(logger_name: str, logging_level: LoggingLevel | None = None) -> logging.Logger:
    """
    Configure how logging should be done.
    
    :param logger_name: The logger name to return.
    :param logging_level: The level of logging to filter. If none, will deduce from "ENV" env variable:
        'dev' will set logging_level to "DEBUG"
        'staging' will set logging_level to "INFO"
        'prod' will set logging_level to "INFO"
    """
    logging.basicConfig(
        # Define the log level for externals (libs) loggers
        level=logging.ERROR,
        handlers=[
            # Write logs to file
            logging.FileHandler(f"logs/{datetime.now().strftime('%d-%m-%Y_%H:%M')}.log"),
            # Allow the logger to also log in console
            logging.StreamHandler(),
        ],
        format="%(asctime)s %(levelname)-8s %(name)-20s -> %(message)s",
        datefmt="%d/%m/%Y %H:%M:%S",
    )
    
    if logging_level is None:
        logging_level = "DEBUG" if get_env_name() == "dev" else "INFO"

    logger = logging.getLogger(logger_name)
    logger.setLevel(logging_level)
    return logger


import unittest

from rabbitmq_helpers import get_messages_from_queue, push_to_queue


class IntegrationTestRabbitMQHelpers(unittest.TestCase):
    """Test class to ensure pushing & reading from queue works as expected."""

    @classmethod
    def setUpClass(cls):
        """Setup once for all tests"""
        cls.queue_name = "updaters_integration_tests"

    def test_push_and_get_message(self):
        """Push a message and then get it from the queue"""

        # Given a message
        test_message = "Integration test message"

        # When pushing the message to the queue
        push_to_queue(self.queue_name, test_message)

        # Then it should be retrieved from the queue
        messages = get_messages_from_queue(self.queue_name, queue_batch_size=1)
        self.assertIn(test_message, messages)

    @classmethod
    def tearDownClass(cls):
        """Cleanup once after all tests"""
        # Optionally, you can clear the queue if required
        while get_messages_from_queue(cls.queue_name, queue_batch_size=1):
            pass

if __name__ == '__main__':
    unittest.main()
